;; listbox helpers

(define (check-range lst number)
  (and (number? number)
       (>= number 0)
       (> (length lst) number)))

(define (safe-list-ref lst number)
  (and (check-range lst number)
       (list-ref lst number)))

(define (safe-list-remove lst number)
  (and (check-range lst number)
       (list-remove lst number)))

(define (unify-item item)
       (if (pair? item)
           (list (car item) (cdr item))
           (list item "")))

;; tree helpers

(define (code->string code)
  (list (string-join (map sure-string (reverse code)) ",")))

(define (make-coordinates lst)
  (let loop ((code (list 0))
             (lst lst)
             (result '()))
    (cond
     ((null? lst) result)
     ((list? (car lst))
        (loop code
              (cdr lst)
              (loop (cons 0 (cons (- (car code) 1) (cdr code)))
                    (car lst) result)))
     (else (loop (cons (+ 1 (car code)) (cdr code))
                 (cdr lst)
                 (append result
                         (code->string code)))))))

(define (tree-map thunk lst)
  (let loop ((lst lst)
             (result '()))
    (cond
     ((null? lst) result)
     ((list? (car lst)) (loop (cdr lst)
                              (append1 result (loop (car lst) '() ))))
     (else (loop (cdr lst) (append1 result (thunk (car lst))))))))


(define (node-ref lst num)
  (cond
   ((list? (car lst)) (node-ref (cdr lst) num))
   ((= num 0) lst)
   (else (node-ref (cdr lst) (- num 1)))))

(define (tree-ref tree num)
  (if (pair? num)
      (let ((node (node-ref tree (car num))))
        (if (pair? (cdr num))
            (tree-ref (cadr node) (cdr num))
            (car node)))
      tree))
